@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 Alberts Muktupāvels
+ * Copyright (C) 2014-2019 Alberts Muktupāvels
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 #include "libautomount-manager/gsd-automount-manager.h"
 #include "libbluetooth-applet/gf-bluetooth-applet.h"
 #include "libclipboard/gf-clipboard.h"
-#include "libdesktop-background/gf-desktop-background.h"
+#include "libdesktop/gf-desktop.h"
 #include "libend-session-dialog/gf-end-session-dialog.h"
 #include "libidle-monitor/flashback-idle-monitor.h"
 #include "libinput-settings/gf-input-settings.h"
@@ -36,6 +36,7 @@
 #include "libnotifications/gf-notifications.h"
 #include "libpolkit/flashback-polkit.h"
 #include "libpower-applet/gf-power-applet.h"
+#include "libroot-background/gf-root-background.h"
 #include "libscreencast/gf-screencast.h"
 #include "libscreensaver/gf-screensaver.h"
 #include "libscreenshot/gf-screenshot.h"
@@ -63,12 +64,13 @@ struct _GfApplication
   GfAudioDeviceSelection  *audio_device_selection;
   GfBluetoothApplet       *bluetooth;
   GfClipboard             *clipboard;
-  GfDesktopBackground     *background;
+  GfDesktop               *desktop;
   GfEndSessionDialog      *dialog;
   GfInputSettings         *input_settings;
   GfInputSources          *input_sources;
   GfNotifications         *notifications;
   GfPowerApplet           *power;
+  GfRootBackground        *root_background;
   GfScreencast            *screencast;
   GfScreensaver           *screensaver;
   GfScreenshot            *screenshot;
@@ -80,6 +82,63 @@ struct _GfApplication
 
 G_DEFINE_TYPE (GfApplication, gf_application, G_TYPE_OBJECT)
 
+typedef struct
+{
+  const char *name;
+  const char *dir;
+  const char *variant;
+  gboolean    has_dark_variant;
+} GfSupportedTheme;
+
+static GfSupportedTheme supported_themes[] =
+  {
+    { "Adwaita", "Adwaita", NULL, TRUE },
+    { "Adwaita-dark", "Adwaita", "dark", FALSE },
+    { "HighContrast", "HighContrast", NULL, FALSE },
+    { "HighContrastInverse", "HighContrast", "inverse", FALSE },
+    { NULL, NULL, FALSE, FALSE }
+  };
+
+static char *
+get_theme_resource (GfSupportedTheme *theme,
+                    gboolean          prefer_dark)
+{
+  char *filename;
+  const char *resource_base;
+  char *resource;
+
+  if (theme->variant != NULL)
+    filename = g_strdup_printf ("gnome-flashback-%s.css", theme->variant);
+  else if (theme->has_dark_variant && prefer_dark)
+    filename = g_strdup ("gnome-flashback-dark.css");
+  else
+    filename = g_strdup ("gnome-flashback.css");
+
+  resource_base = "/org/gnome/gnome-flashback/theme";
+  resource = g_strdup_printf ("%s/%s/%s", resource_base, theme->dir, filename);
+  g_free (filename);
+
+  return resource;
+}
+
+static gboolean
+is_theme_supported (const char        *theme_name,
+                    GfSupportedTheme **theme)
+{
+  int i;
+
+  for (i = 0; supported_themes[i].name != NULL; i++)
+    {
+      if (g_strcmp0 (supported_themes[i].name, theme_name) == 0)
+        {
+          *theme = &supported_themes[i];
+          return TRUE;
+        }
+    }
+
+  return FALSE;
+}
+
 static void
 theme_changed (GtkSettings *settings,
                GParamSpec  *pspec,
@@ -88,9 +147,10 @@ theme_changed (GtkSettings *settings,
   GfApplication *application;
   GdkScreen *screen;
   gchar *theme_name;
-  gboolean dark_theme;
-  guint priority;
+  gboolean prefer_dark;
+  GfSupportedTheme *theme;
   gchar *resource;
+  guint priority;
   GtkCssProvider *css;
 
   application = GF_APPLICATION (user_data);
@@ -102,22 +162,21 @@ theme_changed (GtkSettings *settings,
       g_clear_object (&application->provider);
     }
 
-  g_object_get (settings, "gtk-theme-name", &theme_name, NULL);
-
-  if (g_strcmp0 (theme_name, "Adwaita") != 0 &&
-      g_strcmp0 (theme_name, "HighContrast") != 0)
-    {
-      g_free (theme_name);
-      return;
-    }
-
   g_object_get (settings,
-                "gtk-application-prefer-dark-theme", &dark_theme,
+                "gtk-theme-name", &theme_name,
+                "gtk-application-prefer-dark-theme", &prefer_dark,
                 NULL);
 
-  priority = GTK_STYLE_PROVIDER_PRIORITY_APPLICATION;
-  resource = g_strdup_printf ("/org/gnome/gnome-flashback/theme/%s/gnome-flashback%s.css",
-                              theme_name, dark_theme ? "-dark" : "");
+  if (is_theme_supported (theme_name, &theme))
+    {
+      resource = get_theme_resource (theme, prefer_dark);
+      priority = GTK_STYLE_PROVIDER_PRIORITY_APPLICATION;
+    }
+  else
+    {
+      resource = g_strdup ("/org/gnome/gnome-flashback/theme/fallback.css");
+      priority = GTK_STYLE_PROVIDER_PRIORITY_FALLBACK;
+    }
 
   css = gtk_css_provider_new ();
   application->provider =  GTK_STYLE_PROVIDER (css);
@@ -162,13 +221,14 @@ settings_changed (GSettings   *settings,
   SETTING_CHANGED (a11y_keyboard, "a11y-keyboard", gf_a11y_keyboard_new)
   SETTING_CHANGED (audio_device_selection, "audio-device-selection", gf_audio_device_selection_new)
   SETTING_CHANGED (bluetooth, "bluetooth-applet", gf_bluetooth_applet_new)
-  SETTING_CHANGED (background, "desktop-background", gf_desktop_background_new)
+  SETTING_CHANGED (desktop, "desktop", gf_desktop_new)
   SETTING_CHANGED (clipboard, "clipboard", gf_clipboard_new)
   SETTING_CHANGED (dialog, "end-session-dialog", gf_end_session_dialog_new)
   SETTING_CHANGED (input_settings, "input-settings", gf_input_settings_new)
   SETTING_CHANGED (input_sources, "input-sources", gf_input_sources_new)
   SETTING_CHANGED (notifications, "notifications", gf_notifications_new)
   SETTING_CHANGED (power, "power-applet", gf_power_applet_new)
+  SETTING_CHANGED (root_background, "root-background", gf_root_background_new)
   SETTING_CHANGED (screencast, "screencast", gf_screencast_new)
   SETTING_CHANGED (screensaver, "screensaver", gf_screensaver_new)
   SETTING_CHANGED (screenshot, "screenshot", gf_screenshot_new)
@@ -209,13 +269,14 @@ gf_application_dispose (GObject *object)
   g_clear_object (&application->a11y_keyboard);
   g_clear_object (&application->audio_device_selection);
   g_clear_object (&application->bluetooth);
-  g_clear_object (&application->background);
+  g_clear_object (&application->desktop);
   g_clear_object (&application->clipboard);
   g_clear_object (&application->dialog);
   g_clear_object (&application->input_settings);
   g_clear_object (&application->input_sources);
   g_clear_object (&application->notifications);
   g_clear_object (&application->power);
+  g_clear_object (&application->root_background);
   g_clear_object (&application->screencast);
   g_clear_object (&application->screenshot);
   g_clear_object (&application->screensaver);
@@ -279,8 +340,8 @@ gf_application_init (GfApplication *application)
   g_signal_connect (settings, "notify::gtk-theme-name",
                     G_CALLBACK (theme_changed), application);
 
-  settings_changed (application->settings, NULL, application);
   theme_changed (settings, NULL, application);
+  settings_changed (application->settings, NULL, application);
 
   application->bus_name = g_bus_own_name (G_BUS_TYPE_SESSION,
                                           "org.gnome.Shell",
