@@ -511,14 +511,13 @@ fade_finished_cb (GfInputSourcePopup *popup,
 }
 
 static gboolean
-modifiers_accelerator_activated_cb (GfKeybindings *keybindings,
-                                    gpointer       user_data)
+modifiers_accelerator_activated_cb (GfKeybindings    *keybindings,
+                                    GfKeybindingType  keybinding_type,
+                                    gpointer          user_data)
 {
   GfInputSourceManager *manager;
   guint size;
-  GfInputSource *source;
   GList *keys;
-  guint next_index;
 
   manager = GF_INPUT_SOURCE_MANAGER (user_data);
 
@@ -529,22 +528,48 @@ modifiers_accelerator_activated_cb (GfKeybindings *keybindings,
       return TRUE;
     }
 
-  source = manager->current_source;
-  if (source == NULL)
-    source = (GfInputSource *) g_hash_table_lookup (manager->input_sources,
-                                                    GUINT_TO_POINTER (0));
-
   keys = g_hash_table_get_keys (manager->input_sources);
   keys = g_list_sort (keys, compare_indexes);
 
-  next_index = gf_input_source_get_index (source ) + 1;
-  if (next_index > GPOINTER_TO_UINT (g_list_nth_data (keys, size - 1)))
-    next_index = 0;
+  if (keybinding_type == GF_KEYBINDING_ISO_NEXT_GROUP)
+    {
+      GfInputSource *current_source;
+      GfInputSource *next_source;
+      guint next_index;
 
-  source = (GfInputSource *) g_hash_table_lookup (manager->input_sources,
-                                                  GUINT_TO_POINTER (next_index));
+      current_source = manager->current_source;
+      if (current_source == NULL)
+        current_source = g_hash_table_lookup (manager->input_sources,
+                                              GUINT_TO_POINTER (0));
+
+      next_index = gf_input_source_get_index (current_source ) + 1;
+      if (next_index > GPOINTER_TO_UINT (g_list_nth_data (keys, size - 1)))
+        next_index = 0;
+
+      next_source = g_hash_table_lookup (manager->input_sources,
+                                         GUINT_TO_POINTER (next_index));
+
+      gf_input_source_activate (next_source, TRUE);
+    }
+  else if (keybinding_type == GF_KEYBINDING_ISO_FIRST_GROUP)
+    {
+      GfInputSource *first_source;
+
+      first_source = g_hash_table_lookup (manager->input_sources,
+                                          GUINT_TO_POINTER (0));
+
+      gf_input_source_activate (first_source, TRUE);
+    }
+  else if (keybinding_type == GF_KEYBINDING_ISO_LAST_GROUP)
+    {
+      GfInputSource *last_source;
+
+      last_source = g_hash_table_lookup (manager->input_sources,
+                                         g_list_nth_data (keys, size - 1));
+
+      gf_input_source_activate (last_source, TRUE);
+    }
 
-  gf_input_source_activate (source, TRUE);
   g_list_free (keys);
 
   return TRUE;
@@ -588,11 +613,34 @@ accelerator_activated_cb (GfKeybindings *keybindings,
   gtk_widget_show (manager->popup);
 }
 
+static char *
+get_iso_group_from_xkb_options (gchar **xkb_options)
+{
+  gchar **p;
+  char *option;
+
+  option = NULL;
+
+  for (p = xkb_options; p && *p; ++p)
+    {
+      if (!g_str_has_prefix (*p, "grp:"))
+        continue;
+
+      option = g_strdup (*p + 4);
+      break;
+    }
+
+  return option;
+}
+
 static void
 keybindings_init (GfInputSourceManager *manager)
 {
+  gchar **options;
+  char *iso_group;
+
   manager->wm_keybindings = g_settings_new (DESKTOP_WM_KEYBINDINGS_SCHEMA);
-  manager->keybindings = gf_keybindings_new (TRUE);
+  manager->keybindings = gf_keybindings_new ();
 
   g_signal_connect (manager->wm_keybindings,
                     "changed::" KEY_SWITCH_INPUT_SOURCE,
@@ -604,6 +652,13 @@ keybindings_init (GfInputSourceManager *manager)
                     G_CALLBACK (switch_input_backward_changed_cb),
                     manager);
 
+  options = gf_input_source_settings_get_xkb_options (manager->settings);
+  iso_group = get_iso_group_from_xkb_options (options);
+  g_strfreev (options);
+
+  gf_keybindings_grab_iso_group (manager->keybindings, iso_group);
+  g_free (iso_group);
+
   g_signal_connect (manager->keybindings, "accelerator-activated",
                     G_CALLBACK (accelerator_activated_cb), manager);
   g_signal_connect (manager->keybindings, "modifiers-accelerator-activated",
@@ -1184,11 +1239,16 @@ xkb_options_changed_cb (GfInputSourceSettings *settings,
 {
   GfInputSourceManager *manager;
   gchar **options;
+  char *iso_group;
 
   manager = GF_INPUT_SOURCE_MANAGER (user_data);
 
   options = gf_input_source_settings_get_xkb_options (manager->settings);
 
+  iso_group = get_iso_group_from_xkb_options (options);
+  gf_keybindings_grab_iso_group (manager->keybindings, iso_group);
+  g_free (iso_group);
+
   gf_keyboard_manager_set_xkb_options (manager->keyboard_manager, options);
   g_strfreev (options);
 
@@ -1562,8 +1622,8 @@ gf_input_source_manager_init (GfInputSourceManager *manager)
 {
   manager->keyboard_manager = gf_keyboard_manager_new ();
 
-  keybindings_init (manager);
   input_source_settings_init (manager);
+  keybindings_init (manager);
 }
 
 GfInputSourceManager *
